#include <QApplication>
#include <QGridLayout>
#include <QWidget>
#include <QScreen>
#include <QPainter>
#include <QTimer>
#include <QDebug>
#include <QMoveEvent>

#include <qpa/qplatformwindow.h>

struct formatRect
{
    formatRect(const QRect &r) : m_rect(r) {}

    QRect m_rect;
};

QDebug operator<<(QDebug d, const formatRect &r)
{
    QDebugStateSaver s(d);
    d << r.m_rect.width() << 'x' << r.m_rect.height() << forcesign <<  r.m_rect.x() << r.m_rect.y();
    return d;
}

QDebug operator<<(QDebug d, const QWidget &w)
{
    QDebugStateSaver s(d);
    d.nospace();
    d.noquote();
    d << w.metaObject()->className() << "(\"" << w.objectName() << "\", " << formatRect(w.geometry());
    if (w.isTopLevel())
        d << " [top]";
    if (auto h = w.windowHandle())
    {
        if (auto p = h->handle())
            d << ", native: " << formatRect(p->geometry());
    }
    d << ')';
    return d;
}

static QString title()
{
    return QLatin1String(QT_VERSION_STR) + QLatin1Char(' ') + QGuiApplication::platformName();
}

class EventFilter : public QObject
{
    using QObject::QObject;

    bool eventFilter(QObject *o, QEvent * e) override
    {
        switch (e->type()) {
        case QEvent::MouseButtonPress:
        case QEvent::MouseMove:
            if (static_cast<QMouseEvent *>(e)->buttons())
                qDebug() << o << e;
            break;
        default:
            break;
        }
        return false;
    }
};

int main(int argc, char *argv[])
{
    QApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
    QApplication app(argc, argv);

    QString t = title();
    {
        QDebug d = qDebug().noquote().nospace();
        d << t << '\n';
        for (auto s : QGuiApplication::screens()) {
            d << "  " << s->name() << ' ' << formatRect(s->geometry())
              << " dpr=" << s->devicePixelRatio() << '\n';
        }
    }

    auto childW = new QWidget;
    childW->setObjectName("Child");
    childW->setAutoFillBackground(true);
    childW->setPalette(Qt::red);

    auto mainW = new QWidget;
    mainW->setWindowTitle(t);
    mainW->setObjectName("Main");

    auto l = new QGridLayout(mainW);
    l->setMargin(50);
    l->addWidget(childW);

    mainW->resize(200, 200);
    mainW->show();

    app.installEventFilter(new EventFilter(&app));

    qDebug() << __FUNCTION__ << mainW << childW << '\n';

    QTimer::singleShot(1000, [&] {
        qDebug() << __FUNCTION__ << mainW << *childW << "making native\n";
        childW->setAttribute(Qt::WA_NativeWindow);
        QCoreApplication::processEvents();
        qDebug() << __FUNCTION__ << "after making native" << *childW << '\n';
    });

    return app.exec();
}
